@@ -120,6 +120,12 @@ int main(int argc, char *const argv[])
         return -1;
     }
 
+    if (err = krun_split_irqchip(ctx_id, true)) {
+        errno = -err;
+        perror("Error setting split IRQCHIP property");
+        return -1;
+    }
+
     // Start and enter the microVM. Unless there is some error while creating the microVM
     // this function never returns.
     if (err = krun_start_enter(ctx_id)) {
