@@ -2,9 +2,6 @@
 // SPDX-License-Identifier: Apache-2.0
 
 /// Start of RAM on 64 bit RISC-V.
-#[cfg(not(feature = "efi"))]
-pub const DRAM_MEM_START: u64 = 0x8000_0000; // 2 GB.
-#[cfg(feature = "efi")]
 pub const DRAM_MEM_START: u64 = 0x4000_0000; // 1 GB.
 /// The maximum addressable RAM address.
 pub const DRAM_MEM_END: u64 = 0x00FF_8000_0000; // 1024 - 2 = 1022 GB.
@@ -31,10 +28,13 @@ pub const APLIC_START: u64 = 0;
 pub const IMSIC_START: u64 = 0x0400_0000;
 
 /// Below this address will reside the AIA, above this address will reside the MMIO devices.
-#[cfg(not(feature = "efi"))]
-pub const MAPPED_IO_START: u64 = 1 << 30; // 1 GB
-#[cfg(feature = "efi")]
 pub const MAPPED_IO_START: u64 = 0x0a00_0000;
 
-#[cfg(feature = "efi")]
+/// The address to put the SMBIOS contents, if present.
 pub const SMBIOS_START: u64 = 0x4000_F000;
+
+/// Where the PC register will point after a reset.
+pub const RESET_VECTOR: u64 = 0;
+
+/// The address to load the firmware, if present.
+pub const FIRMWARE_START: u64 = 0;
