@@ -14,7 +14,7 @@ use std::fmt;
 use std::io;
 
 mod bus;
-#[cfg(target_arch = "aarch64")]
+#[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
 pub mod fdt;
 pub mod legacy;
 pub mod virtio;
@@ -44,7 +44,7 @@ pub enum DeviceType {
     #[cfg(target_arch = "aarch64")]
     Gpio,
     /// Device Type: Serial.
-    #[cfg(target_arch = "aarch64")]
+    #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     Serial,
     /// Device Type: RTC.
     #[cfg(target_arch = "aarch64")]
