@@ -7,23 +7,19 @@
 
 use std::collections::HashMap;
 use std::path::PathBuf;
-use std::result;
-use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
 
 use utils::byte_order;
 use utils::eventfd::EventFd;
 use vm_memory::GuestMemoryMmap;
 
-use super::super::super::Error as DeviceError;
 use super::super::{
     ActivateError, ActivateResult, DeviceState, Queue as VirtQueue, VirtioDevice, VsockError,
-    VIRTIO_MMIO_INT_VRING,
 };
 use super::muxer::VsockMuxer;
 use super::packet::VsockPacket;
 use super::{defs, defs::uapi};
-use crate::legacy::IrqChip;
+use crate::virtio::InterruptTransport;
 
 pub(crate) const RXQ_INDEX: usize = 0;
 pub(crate) const TXQ_INDEX: usize = 1;
@@ -46,12 +42,8 @@ pub struct Vsock {
     pub(crate) queue_events: Vec<EventFd>,
     pub(crate) avail_features: u64,
     pub(crate) acked_features: u64,
-    pub(crate) interrupt_status: Arc<AtomicUsize>,
-    pub(crate) interrupt_evt: EventFd,
     pub(crate) activate_evt: EventFd,
     pub(crate) device_state: DeviceState,
-    intc: Option<IrqChip>,
-    irq_line: Option<u32>,
 }
 
 impl Vsock {
@@ -70,32 +62,18 @@ impl Vsock {
         let queue_tx = Arc::new(Mutex::new(queues[TXQ_INDEX].clone()));
         let queue_rx = Arc::new(Mutex::new(queues[RXQ_INDEX].clone()));
 
-        let interrupt_evt =
-            EventFd::new(utils::eventfd::EFD_NONBLOCK).map_err(VsockError::EventFd)?;
-        let interrupt_status = Arc::new(AtomicUsize::new(0));
-
         Ok(Vsock {
             cid,
-            muxer: VsockMuxer::new(
-                cid,
-                host_port_map,
-                interrupt_evt.try_clone().unwrap(),
-                interrupt_status.clone(),
-                unix_ipc_port_map,
-            ),
+            muxer: VsockMuxer::new(cid, host_port_map, unix_ipc_port_map),
             queue_rx,
             queue_tx,
             queues,
             queue_events,
             avail_features: AVAIL_FEATURES,
             acked_features: 0,
-            interrupt_status,
-            interrupt_evt,
             activate_evt: EventFd::new(utils::eventfd::EFD_NONBLOCK)
                 .map_err(VsockError::EventFd)?,
             device_state: DeviceState::Inactive,
-            intc: None,
-            irq_line: None,
         })
     }
 
@@ -116,35 +94,17 @@ impl Vsock {
         defs::VSOCK_DEV_ID
     }
 
-    pub fn set_intc(&mut self, intc: IrqChip) {
-        self.intc = Some(intc);
-    }
-
     pub fn cid(&self) -> u64 {
         self.cid
     }
 
-    /// Signal the guest driver that we've used some virtio buffers that it had previously made
-    /// available.
-    pub fn signal_used_queue(&self) -> result::Result<(), DeviceError> {
-        debug!("vsock: raising IRQ");
-        self.interrupt_status
-            .fetch_or(VIRTIO_MMIO_INT_VRING as usize, Ordering::SeqCst);
-        if let Some(intc) = &self.intc {
-            intc.lock()
-                .unwrap()
-                .set_irq(self.irq_line, Some(&self.interrupt_evt))?;
-        }
-        Ok(())
-    }
-
     /// Walk the driver-provided RX queue buffers and attempt to fill them up with any data that we
     /// have pending. Return `true` if descriptors have been added to the used ring, and `false`
     /// otherwise.
     pub fn process_stream_rx(&mut self) -> bool {
         debug!("vsock: process_stream_rx()");
         let mem = match self.device_state {
-            DeviceState::Activated(ref mem) => mem,
+            DeviceState::Activated(ref mem, _) => mem,
             // This should never happen, it's been already validated in the event handler.
             DeviceState::Inactive => unreachable!(),
         };
@@ -167,7 +127,7 @@ impl Vsock {
                     }
                 }
                 Err(e) => {
-                    warn!("vsock: RX queue error: {:?}", e);
+                    warn!("vsock: RX queue error: {e:?}");
                     0
                 }
             };
@@ -175,7 +135,7 @@ impl Vsock {
             debug!("vsock: process_rx: something to queue");
             have_used = true;
             if let Err(e) = queue_rx.add_used(mem, head.index, used_len) {
-                error!("failed to add used elements to the queue: {:?}", e);
+                error!("failed to add used elements to the queue: {e:?}");
             }
         }
 
@@ -187,7 +147,7 @@ impl Vsock {
     pub fn process_stream_tx(&mut self) -> bool {
         debug!("vsock::process_stream_tx()");
         let mem = match self.device_state {
-            DeviceState::Activated(ref mem) => mem,
+            DeviceState::Activated(ref mem, _) => mem,
             // This should never happen, it's been already validated in the event handler.
             DeviceState::Inactive => unreachable!(),
         };
@@ -199,10 +159,10 @@ impl Vsock {
             let pkt = match VsockPacket::from_tx_virtq_head(&head) {
                 Ok(pkt) => pkt,
                 Err(e) => {
-                    error!("vsock: error reading TX packet: {:?}", e);
+                    error!("vsock: error reading TX packet: {e:?}");
                     have_used = true;
                     if let Err(e) = queue_tx.add_used(mem, head.index, 0) {
-                        error!("failed to add used elements to the queue: {:?}", e);
+                        error!("failed to add used elements to the queue: {e:?}");
                     }
                     continue;
                 }
@@ -224,7 +184,7 @@ impl Vsock {
 
             have_used = true;
             if let Err(e) = queue_tx.add_used(mem, head.index, 0) {
-                error!("failed to add used elements to the queue: {:?}", e);
+                error!("failed to add used elements to the queue: {e:?}");
             }
         }
 
@@ -249,6 +209,10 @@ impl VirtioDevice for Vsock {
         uapi::VIRTIO_ID_VSOCK
     }
 
+    fn device_name(&self) -> &str {
+        "vsock"
+    }
+
     fn queues(&self) -> &[VirtQueue] {
         &self.queues
     }
@@ -261,19 +225,6 @@ impl VirtioDevice for Vsock {
         &self.queue_events
     }
 
-    fn interrupt_evt(&self) -> &EventFd {
-        &self.interrupt_evt
-    }
-
-    fn interrupt_status(&self) -> Arc<AtomicUsize> {
-        self.interrupt_status.clone()
-    }
-
-    fn set_irq_line(&mut self, irq: u32) {
-        debug!("SET_IRQ_LINE (VSOCK)={}", irq);
-        self.irq_line = Some(irq);
-    }
-
     fn read_config(&self, offset: u64, data: &mut [u8]) {
         match offset {
             0 if data.len() == 8 => byte_order::write_le_u64(data, self.cid()),
@@ -299,7 +250,7 @@ impl VirtioDevice for Vsock {
         );
     }
 
-    fn activate(&mut self, mem: GuestMemoryMmap) -> ActivateResult {
+    fn activate(&mut self, mem: GuestMemoryMmap, interrupt: InterruptTransport) -> ActivateResult {
         if self.queues.len() != defs::NUM_QUEUES {
             error!(
                 "Cannot perform activate. Expected {} queue(s), got {}",
@@ -316,22 +267,15 @@ impl VirtioDevice for Vsock {
 
         self.queue_tx = Arc::new(Mutex::new(self.queues[TXQ_INDEX].clone()));
         self.queue_rx = Arc::new(Mutex::new(self.queues[RXQ_INDEX].clone()));
-        self.muxer.activate(
-            mem.clone(),
-            self.queue_rx.clone(),
-            self.intc.clone(),
-            self.irq_line,
-        );
+        self.muxer
+            .activate(mem.clone(), self.queue_rx.clone(), interrupt.clone());
 
-        self.device_state = DeviceState::Activated(mem);
+        self.device_state = DeviceState::Activated(mem, interrupt);
 
         Ok(())
     }
 
     fn is_activated(&self) -> bool {
-        match self.device_state {
-            DeviceState::Inactive => false,
-            DeviceState::Activated(_) => true,
-        }
+        self.device_state.is_activated()
     }
 }
