@@ -3,14 +3,15 @@ use super::{
     proxy::{ProxyRemoval, RecvPkt},
 };
 
+use nix::errno::Errno;
 use nix::fcntl::{fcntl, FcntlArg, OFlag};
 use nix::sys::socket::{
-    accept, bind, connect, listen, recv, send, shutdown, socket, AddressFamily, MsgFlags, Shutdown,
-    SockFlag, SockType, UnixAddr,
+    accept, bind, connect, listen, recv, send, shutdown, socket, AddressFamily, Backlog, MsgFlags,
+    Shutdown, SockFlag, SockType, UnixAddr,
 };
-use nix::unistd::close;
 use std::collections::HashMap;
 use std::num::Wrapping;
+use std::os::fd::{FromRawFd, OwnedFd};
 use std::os::unix::io::{AsRawFd, RawFd};
 use std::path::PathBuf;
 use std::sync::{Arc, Mutex};
@@ -29,7 +30,7 @@ use vm_memory::GuestMemoryMmap;
 pub struct UnixProxy {
     id: u64,
     cid: u64,
-    fd: RawFd,
+    fd: OwnedFd,
     pub status: ProxyStatus,
     mem: GuestMemoryMmap,
     queue: Arc<Mutex<VirtQueue>>,
@@ -46,7 +47,7 @@ pub struct UnixProxy {
     rx_cnt: Wrapping<u32>,
 }
 
-fn proxy_fd_create(id: u64) -> Result<RawFd, ProxyError> {
+fn proxy_fd_create(id: u64) -> Result<OwnedFd, ProxyError> {
     let fd = socket(
         AddressFamily::Unix,
         SockType::Stream,
@@ -56,10 +57,10 @@ fn proxy_fd_create(id: u64) -> Result<RawFd, ProxyError> {
     .map_err(ProxyError::CreatingSocket)?;
 
     // macOS forces us to do this here instead of just using SockFlag::SOCK_NONBLOCK above.
-    match fcntl(fd, FcntlArg::F_GETFL) {
+    match fcntl(&fd, FcntlArg::F_GETFL) {
         Ok(flags) => match OFlag::from_bits(flags) {
             Some(flags) => {
-                if let Err(e) = fcntl(fd, FcntlArg::F_SETFL(flags | OFlag::O_NONBLOCK)) {
+                if let Err(e) = fcntl(&fd, FcntlArg::F_SETFL(flags | OFlag::O_NONBLOCK)) {
                     warn!("error switching to non-blocking: id={id}, err={e}");
                 }
             }
@@ -74,7 +75,7 @@ fn proxy_fd_create(id: u64) -> Result<RawFd, ProxyError> {
         let option_value: libc::c_int = 1;
         unsafe {
             libc::setsockopt(
-                fd,
+                fd.as_raw_fd(),
                 libc::SOL_SOCKET,
                 libc::SO_NOSIGPIPE,
                 &option_value as *const _ as *const libc::c_void,
@@ -127,7 +128,7 @@ impl UnixProxy {
         cid: u64,
         local_port: u32,
         peer_port: u32,
-        fd: RawFd,
+        fd: OwnedFd,
         mem: GuestMemoryMmap,
         queue: Arc<Mutex<VirtQueue>>,
         rxq: Arc<Mutex<MuxerRxQ>>,
@@ -156,10 +157,10 @@ impl UnixProxy {
 
     fn switch_to_connected(&mut self) {
         self.status = ProxyStatus::Connected;
-        match fcntl(self.fd, FcntlArg::F_GETFL) {
+        match fcntl(&self.fd, FcntlArg::F_GETFL) {
             Ok(flags) => match OFlag::from_bits(flags) {
                 Some(flags) => {
-                    if let Err(e) = fcntl(self.fd, FcntlArg::F_SETFL(flags & !OFlag::O_NONBLOCK)) {
+                    if let Err(e) = fcntl(&self.fd, FcntlArg::F_SETFL(flags & !OFlag::O_NONBLOCK)) {
                         warn!("error switching to blocking: id={}, err={}", self.id, e);
                     }
                 }
@@ -218,7 +219,11 @@ impl UnixProxy {
                 return RecvPkt::WaitForCredit;
             }
 
-            match recv(self.fd, &mut buf[..max_len], MsgFlags::MSG_DONTWAIT) {
+            match recv(
+                self.fd.as_raw_fd(),
+                &mut buf[..max_len],
+                MsgFlags::MSG_DONTWAIT,
+            ) {
                 Ok(cnt) => {
                     debug!("vsock: unix: recv cnt={cnt}");
                     if cnt > 0 {
@@ -320,7 +325,7 @@ impl Proxy for UnixProxy {
 
         let addr = UnixAddr::new(&self.path).unwrap();
 
-        let result = match connect(self.fd, &addr) {
+        let result = match connect(self.fd.as_raw_fd(), &addr) {
             Ok(()) => {
                 debug!("vsock: connect: Connected");
                 self.switch_to_connected();
@@ -334,18 +339,18 @@ impl Proxy for UnixProxy {
             Err(e) => {
                 debug!("vsock: UnixProxy: Error connecting: {e}");
                 #[cfg(target_os = "macos")]
-                let errno = -linux_errno_raw(nix::errno::errno());
+                let errno = -linux_errno_raw(Errno::last_raw());
                 #[cfg(target_os = "linux")]
-                let errno = -nix::errno::errno();
+                let errno = -Errno::last_raw();
                 errno
             }
         };
 
         if self.status == ProxyStatus::Connecting {
-            update.polling = Some((self.id, self.fd, EventSet::IN | EventSet::OUT));
+            update.polling = Some((self.id, self.fd.as_raw_fd(), EventSet::IN | EventSet::OUT));
         } else {
             if self.status == ProxyStatus::Connected {
-                update.polling = Some((self.id, self.fd, EventSet::IN));
+                update.polling = Some((self.id, self.fd.as_raw_fd(), EventSet::IN));
             }
             self.push_connect_rsp(result);
         }
@@ -392,7 +397,7 @@ impl Proxy for UnixProxy {
             #[cfg(target_os = "linux")]
             let flags = MsgFlags::MSG_NOSIGNAL;
 
-            match send(self.fd, buf, flags) {
+            match send(self.fd.as_raw_fd(), buf, flags) {
                 Ok(sent) => {
                     if sent != buf.len() {
                         error!("couldn't set everything: buf={}, sent={}", buf.len(), sent);
@@ -467,7 +472,7 @@ impl Proxy for UnixProxy {
         self.status = ProxyStatus::Connected;
 
         ProxyUpdate {
-            polling: Some((self.id, self.fd, EventSet::IN)),
+            polling: Some((self.id, self.fd.as_raw_fd(), EventSet::IN)),
             ..Default::default()
         }
     }
@@ -500,7 +505,7 @@ impl Proxy for UnixProxy {
         self.switch_to_connected();
 
         ProxyUpdate {
-            polling: Some((self.id, self.fd, EventSet::IN)),
+            polling: Some((self.id, self.fd.as_raw_fd(), EventSet::IN)),
             ..Default::default()
         }
     }
@@ -521,7 +526,7 @@ impl Proxy for UnixProxy {
             Shutdown::Write
         };
 
-        if let Err(e) = shutdown(self.fd, how) {
+        if let Err(e) = shutdown(self.fd.as_raw_fd(), how) {
             warn!("error sending shutdown to socket: {e}");
         }
     }
@@ -552,7 +557,7 @@ impl Proxy for UnixProxy {
             }
 
             self.status = ProxyStatus::Closed;
-            update.polling = Some((self.id, self.fd, EventSet::empty()));
+            update.polling = Some((self.id, self.fd.as_raw_fd(), EventSet::empty()));
             update.signal_queue = true;
             update.remove_proxy = ProxyRemoval::Deferred;
 
@@ -583,11 +588,11 @@ impl Proxy for UnixProxy {
 
                     self.push_reset();
                     update.signal_queue = true;
-                    update.polling = Some((self.id(), self.fd, EventSet::empty()));
+                    update.polling = Some((self.id(), self.fd.as_raw_fd(), EventSet::empty()));
                     return update;
                 } else if self.status == ProxyStatus::WaitingCreditUpdate {
                     debug!("process_event: WaitingCreditUpdate");
-                    update.polling = Some((self.id(), self.fd, EventSet::empty()));
+                    update.polling = Some((self.id(), self.fd.as_raw_fd(), EventSet::empty()));
                 }
             } else {
                 debug!(
@@ -603,7 +608,7 @@ impl Proxy for UnixProxy {
                 self.switch_to_connected();
                 self.push_connect_rsp(0);
                 update.signal_queue = true;
-                update.polling = Some((self.id(), self.fd, EventSet::IN));
+                update.polling = Some((self.id(), self.fd.as_raw_fd(), EventSet::IN));
             } else {
                 error!("vsock::tcp: EventSet::OUT while not connecting");
             }
@@ -615,21 +620,13 @@ impl Proxy for UnixProxy {
 
 impl AsRawFd for UnixProxy {
     fn as_raw_fd(&self) -> RawFd {
-        self.fd
-    }
-}
-
-impl Drop for UnixProxy {
-    fn drop(&mut self) {
-        if let Err(e) = close(self.fd) {
-            warn!("error closing proxy fd: {e}");
-        }
+        self.fd.as_raw_fd()
     }
 }
 
 pub struct UnixAcceptorProxy {
     id: u64,
-    fd: RawFd,
+    fd: OwnedFd,
     peer_port: u32,
 }
 
@@ -643,11 +640,12 @@ impl UnixAcceptorProxy {
         )
         .map_err(ProxyError::CreatingSocket)?;
         bind(
-            fd,
+            fd.as_raw_fd(),
             &UnixAddr::new(path).map_err(ProxyError::CreatingSocket)?,
         )
         .map_err(ProxyError::CreatingSocket)?;
-        listen(fd, 5).map_err(ProxyError::CreatingSocket)?;
+        listen(&fd, Backlog::new(5).map_err(ProxyError::CreatingSocket)?)
+            .map_err(ProxyError::CreatingSocket)?;
         Ok(UnixAcceptorProxy { id, fd, peer_port })
     }
 }
@@ -696,15 +694,17 @@ impl Proxy for UnixAcceptorProxy {
 
         if evset.contains(EventSet::HANG_UP) {
             debug!("process_event: HANG_UP");
-            update.polling = Some((self.id, self.fd, EventSet::empty()));
+            update.polling = Some((self.id, self.fd.as_raw_fd(), EventSet::empty()));
             update.signal_queue = true;
             update.remove_proxy = ProxyRemoval::Deferred;
             return update;
         }
         if evset.contains(EventSet::IN) {
-            match accept(self.fd) {
+            match accept(self.fd.as_raw_fd()) {
                 Ok(accept_fd) => {
-                    update.new_proxy = Some((self.peer_port, accept_fd, NewProxyType::Unix));
+                    // Safe because we've just obtained the FD from the `accept` call above.
+                    let new_fd = unsafe { OwnedFd::from_raw_fd(accept_fd) };
+                    update.new_proxy = Some((self.peer_port, new_fd, NewProxyType::Unix));
                 }
                 Err(e) => warn!("error accepting connection: id={}, err={}", self.id, e),
             };
@@ -716,14 +716,6 @@ impl Proxy for UnixAcceptorProxy {
 
 impl AsRawFd for UnixAcceptorProxy {
     fn as_raw_fd(&self) -> RawFd {
-        self.fd
-    }
-}
-
-impl Drop for UnixAcceptorProxy {
-    fn drop(&mut self) {
-        if let Err(e) = close(self.fd) {
-            warn!("error closing proxy fd: {e}");
-        }
+        self.fd.as_raw_fd()
     }
 }
