@@ -13,6 +13,7 @@ use devices::virtio::CacheType;
 use env_logger::{Env, Target};
 #[cfg(feature = "gpu")]
 use krun_display::DisplayBackend;
+
 use libc::c_char;
 #[cfg(feature = "net")]
 use libc::c_int;
@@ -31,26 +32,28 @@ use std::ffi::{c_void, CStr};
 use std::fs::File;
 #[cfg(target_os = "linux")]
 use std::os::fd::AsRawFd;
+#[cfg(feature = "input")]
+use std::os::fd::BorrowedFd;
 use std::os::fd::{FromRawFd, RawFd};
 use std::path::PathBuf;
 use std::slice;
 use std::sync::atomic::{AtomicI32, Ordering};
-#[cfg(not(feature = "efi"))]
 use std::sync::LazyLock;
 use std::sync::Mutex;
 use utils::eventfd::EventFd;
-use vmm::resources::VmResources;
+use vmm::resources::{ConsoleConfig, ConsoleType, VmResources};
 #[cfg(feature = "blk")]
 use vmm::vmm_config::block::{BlockDeviceConfig, BlockRootConfig};
-use vmm::vmm_config::boot_source::{BootSourceConfig, DEFAULT_KERNEL_CMDLINE};
 #[cfg(not(feature = "tee"))]
 use vmm::vmm_config::external_kernel::{ExternalKernel, KernelFormat};
 #[cfg(not(feature = "tee"))]
+use vmm::vmm_config::firmware::FirmwareConfig;
+#[cfg(not(feature = "tee"))]
 use vmm::vmm_config::fs::FsDeviceConfig;
-#[cfg(not(feature = "efi"))]
 use vmm::vmm_config::kernel_bundle::KernelBundle;
 #[cfg(feature = "tee")]
 use vmm::vmm_config::kernel_bundle::{InitrdBundle, QbootBundle};
+use vmm::vmm_config::kernel_cmdline::{KernelCmdlineConfig, DEFAULT_KERNEL_CMDLINE};
 use vmm::vmm_config::machine_config::VmConfig;
 #[cfg(feature = "net")]
 use vmm::vmm_config::net::NetworkInterfaceConfig;
@@ -61,6 +64,8 @@ use nitro::enclaves::NitroEnclave;
 
 #[cfg(feature = "gpu")]
 use devices::virtio::display::{DisplayInfoEdid, PhysicalSize, MAX_DISPLAYS};
+#[cfg(feature = "input")]
+use krun_input::{InputConfigBackend, InputEventProviderBackend};
 #[cfg(feature = "nitro")]
 use nitro_enclaves::launch::StartFlags;
 
@@ -76,17 +81,15 @@ const KRUNFW_NAME: &str = "libkrunfw.so.4";
 const KRUNFW_NAME: &str = "libkrunfw-sev.so.4";
 #[cfg(all(target_os = "linux", feature = "tdx"))]
 const KRUNFW_NAME: &str = "libkrunfw-tdx.so.4";
-#[cfg(all(target_os = "macos", not(feature = "efi")))]
+#[cfg(target_os = "macos")]
 const KRUNFW_NAME: &str = "libkrunfw.4.dylib";
 
 // Path to the init binary to be executed inside the VM.
 const INIT_PATH: &str = "/init.krun";
 
-#[cfg(not(feature = "efi"))]
 static KRUNFW: LazyLock<Option<libloading::Library>> =
     LazyLock::new(|| unsafe { libloading::Library::new(KRUNFW_NAME).ok() });
 
-#[cfg(not(feature = "efi"))]
 pub struct KrunfwBindings {
     get_kernel: libloading::Symbol<
         'static,
@@ -98,7 +101,6 @@ pub struct KrunfwBindings {
     get_qboot: libloading::Symbol<'static, unsafe extern "C" fn(*mut size_t) -> *mut c_char>,
 }
 
-#[cfg(not(feature = "efi"))]
 impl KrunfwBindings {
     fn load_bindings() -> Result<KrunfwBindings, libloading::Error> {
         let krunfw = match KRUNFW.as_ref() {
@@ -130,7 +132,6 @@ enum LegacyNetworkConfig {
 
 #[derive(Default)]
 struct ContextConfig {
-    #[cfg(not(feature = "efi"))]
     krunfw: Option<KrunfwBindings>,
     vmr: VmResources,
     workdir: Option<String>,
@@ -206,10 +207,10 @@ impl ContextConfig {
             Some(block_root) => {
                 let mut res = format!("KRUN_BLOCK_ROOT_DEVICE={}", block_root.device);
                 if let Some(fstype) = &block_root.fstype {
-                    res += &format!(" KRUN_BLOCK_ROOT_FSTYPE={}", fstype);
+                    res += &format!(" KRUN_BLOCK_ROOT_FSTYPE={fstype}");
                 }
                 if let Some(options) = &block_root.options {
-                    res += &format!(" KRUN_BLOCK_ROOT_OPTIONS={}", options);
+                    res += &format!(" KRUN_BLOCK_ROOT_OPTIONS={options}");
                 }
                 res
             }
@@ -463,15 +464,14 @@ pub unsafe extern "C" fn krun_init_log(target: RawFd, level: u32, style: u32, op
 
 #[no_mangle]
 pub extern "C" fn krun_create_ctx() -> i32 {
-    let ctx_cfg = {
-        let shutdown_efd = if cfg!(feature = "efi") {
-            Some(EventFd::new(utils::eventfd::EFD_NONBLOCK).unwrap())
-        } else {
-            None
-        };
+    let shutdown_efd = if cfg!(target_arch = "aarch64") && cfg!(target_os = "macos") {
+        Some(EventFd::new(utils::eventfd::EFD_NONBLOCK).unwrap())
+    } else {
+        None
+    };
 
+    let ctx_cfg = {
         ContextConfig {
-            #[cfg(not(feature = "efi"))]
             krunfw: KrunfwBindings::new(),
             shutdown_efd,
             ..Default::default()
@@ -1407,6 +1407,91 @@ pub extern "C" fn krun_set_display_backend(
     KRUN_SUCCESS
 }
 
+#[cfg(not(feature = "input"))]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub extern "C" fn krun_add_input_device(
+    _ctx_id: u32,
+    _config_backend: *const c_void,
+    _config_backend_size: size_t,
+    _event_provider_backend: *const c_void,
+    _event_provider_backend_size: size_t,
+) -> i32 {
+    -libc::ENOTSUP
+}
+
+#[cfg(feature = "input")]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub extern "C" fn krun_add_input_device_fd(ctx_id: u32, input_fd: i32) -> i32 {
+    use devices::virtio::input::passthrough::PassthroughInputBackend;
+    use krun_input::{IntoInputConfig, IntoInputEvents};
+
+    if input_fd < 0 {
+        return -libc::EINVAL;
+    }
+    // TODO: currently we let the fd (and it's Box allocation) live forever, we should eventually fix
+    //       this
+    let input_fd = unsafe {
+        // SAFETY: The user provided fd should be valid. Its lifetime is 'static because it will
+        //         exist until libkrun _exits the process
+        BorrowedFd::borrow_raw(input_fd)
+    };
+    let borrowed_fd: &'static BorrowedFd<'static> = Box::leak(Box::new(input_fd));
+
+    let config_backend = PassthroughInputBackend::into_input_config(Some(borrowed_fd));
+    let events_backend = PassthroughInputBackend::into_input_events(Some(borrowed_fd));
+
+    with_cfg(ctx_id, |cfg| {
+        cfg.vmr
+            .input_backends
+            .push((config_backend, events_backend));
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(feature = "input")]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_input_device(
+    ctx_id: u32,
+    config_backend: *const InputConfigBackend<'static>,
+    config_backend_size: size_t,
+    event_provider_backend: *const InputEventProviderBackend<'static>,
+    event_provider_backend_size: size_t,
+) -> i32 {
+    if config_backend.is_null() || event_provider_backend.is_null() {
+        return -libc::EINVAL;
+    }
+
+    if config_backend_size < size_of::<InputConfigBackend>()
+        || event_provider_backend_size < size_of::<InputEventProviderBackend>()
+    {
+        return -libc::EINVAL;
+    }
+
+    let config_backend = unsafe { *config_backend };
+    let events_backend = unsafe { *event_provider_backend };
+
+    if !config_backend.verify() || !events_backend.verify() {
+        return -libc::EINVAL;
+    }
+
+    with_cfg(ctx_id, |cfg| {
+        cfg.vmr
+            .input_backends
+            .push((config_backend, events_backend));
+        KRUN_SUCCESS
+    })
+}
+
+#[cfg(not(feature = "input"))]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_input_device_fd(_ctx_id: u32, _input_fd: i32) -> i32 {
+    -libc::ENOTSUP
+}
+
 #[cfg(feature = "gpu")]
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
@@ -1638,6 +1723,39 @@ pub unsafe extern "C" fn krun_check_nested_virt() -> i32 {
     -libc::EOPNOTSUPP
 }
 
+/// Gets the maximum number of vCPUs supported by the hypervisor.
+///
+/// Returns the maximum number of vCPUs that can be created by this hypervisor,
+/// or a negative error code on failure.
+#[cfg(any(target_os = "macos", target_os = "linux"))]
+#[no_mangle]
+pub extern "C" fn krun_get_max_vcpus() -> i32 {
+    #[cfg(target_os = "macos")]
+    {
+        use hvf::bindings::{hv_vm_get_max_vcpu_count, HV_SUCCESS};
+        let mut max_vcpu_count: u32 = 0;
+        let ret = unsafe { hv_vm_get_max_vcpu_count(&mut max_vcpu_count as *mut u32) };
+        if ret == HV_SUCCESS {
+            max_vcpu_count as i32
+        } else {
+            error!("Error retrieving max vcpu count: {ret:?}");
+            -libc::EINVAL
+        }
+    }
+
+    #[cfg(target_os = "linux")]
+    {
+        use kvm_ioctls::Kvm;
+        match Kvm::new() {
+            Ok(kvm) => kvm.get_max_vcpus() as i32,
+            Err(e) => {
+                error!("Error retrieving max vcpu count: {e:?}");
+                -libc::EINVAL
+            }
+        }
+    }
+}
+
 #[allow(clippy::missing_safety_doc)]
 #[no_mangle]
 pub extern "C" fn krun_split_irqchip(ctx_id: u32, enable: bool) -> i32 {
@@ -1846,7 +1964,29 @@ pub unsafe extern "C" fn krun_set_kernel(
     KRUN_SUCCESS
 }
 
-#[cfg(not(feature = "efi"))]
+#[cfg(not(feature = "tee"))]
+#[allow(clippy::format_collect)]
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_set_firmware(ctx_id: u32, c_firmware_path: *const c_char) -> i32 {
+    let path = match CStr::from_ptr(c_firmware_path).to_str() {
+        Ok(path) => PathBuf::from(path),
+        Err(e) => {
+            error!("Error parsing firmware_path: {e:?}");
+            return -libc::EINVAL;
+        }
+    };
+
+    let firmware_config = FirmwareConfig { path };
+
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => ctx_cfg.get_mut().vmr.set_firmware_config(firmware_config),
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
 unsafe fn load_krunfw_payload(
     krunfw: &KrunfwBindings,
     vmr: &mut VmResources,
@@ -2049,6 +2189,92 @@ pub unsafe extern "C" fn krun_set_root_disk_remount(
     KRUN_SUCCESS
 }
 
+#[no_mangle]
+pub extern "C" fn krun_disable_implicit_console(ctx_id: u32) -> i32 {
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.vmr.disable_implicit_console = true;
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_virtio_console_default(
+    ctx_id: u32,
+    input_fd: libc::c_int,
+    output_fd: libc::c_int,
+    err_fd: libc::c_int,
+) -> i32 {
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.vmr.consoles.entry(ConsoleType::Virtio).or_default();
+
+            // safe to unwrap since we inserted an empty Vec if the key didn't exist
+            let consoles = cfg.vmr.consoles.get_mut(&ConsoleType::Virtio).unwrap();
+            consoles.push(ConsoleConfig {
+                output_path: None,
+                input_fd,
+                output_fd,
+                err_fd,
+            });
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_add_serial_console_default(
+    ctx_id: u32,
+    input_fd: libc::c_int,
+    output_fd: libc::c_int,
+) -> i32 {
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.vmr.consoles.entry(ConsoleType::Serial).or_default();
+
+            // safe to unwrap since we inserted an empty Vec if the key didn't exist
+            let consoles = cfg.vmr.consoles.get_mut(&ConsoleType::Serial).unwrap();
+            consoles.push(ConsoleConfig {
+                output_path: None,
+                input_fd,
+                output_fd,
+                err_fd: -1,
+            });
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
+#[allow(clippy::missing_safety_doc)]
+#[no_mangle]
+pub unsafe extern "C" fn krun_set_kernel_console(ctx_id: u32, console_id: *const c_char) -> i32 {
+    let console_id = match CStr::from_ptr(console_id).to_str() {
+        Ok(id) => id.to_string(),
+        Err(_) => return -libc::EINVAL,
+    };
+    match CTX_MAP.lock().unwrap().entry(ctx_id) {
+        Entry::Occupied(mut ctx_cfg) => {
+            let cfg = ctx_cfg.get_mut();
+            cfg.vmr.kernel_console = Some(console_id);
+        }
+        Entry::Vacant(_) => return -libc::ENOENT,
+    }
+
+    KRUN_SUCCESS
+}
+
 #[no_mangle]
 #[allow(unreachable_code)]
 pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
@@ -2077,8 +2303,11 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         None => return -libc::ENOENT,
     };
 
-    #[cfg(not(feature = "efi"))]
-    if ctx_cfg.vmr.external_kernel.is_none() && ctx_cfg.vmr.kernel_bundle.is_none() {
+    if ctx_cfg.vmr.external_kernel.is_none()
+        && ctx_cfg.vmr.kernel_bundle.is_none()
+        && ctx_cfg.vmr.firmware_config.is_none()
+        && cfg!(not(feature = "efi"))
+    {
         if let Some(ref krunfw) = ctx_cfg.krunfw {
             if let Err(err) = unsafe { load_krunfw_payload(krunfw, &mut ctx_cfg.vmr) } {
                 eprintln!("Can't load libkrunfw symbols: {err}");
@@ -2115,9 +2344,9 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
         return -libc::EINVAL;
     }
 
-    let boot_source = BootSourceConfig {
-        kernel_cmdline_prolog: Some(format!("{} init={}", DEFAULT_KERNEL_CMDLINE, INIT_PATH)),
-        kernel_cmdline_krun_env: Some(format!(
+    let kernel_cmdline = KernelCmdlineConfig {
+        prolog: Some(format!("{DEFAULT_KERNEL_CMDLINE} init={INIT_PATH}")),
+        krun_env: Some(format!(
             " {} {} {} {} {}",
             ctx_cfg.get_exec_path(),
             ctx_cfg.get_workdir(),
@@ -2125,10 +2354,10 @@ pub extern "C" fn krun_start_enter(ctx_id: u32) -> i32 {
             ctx_cfg.get_rlimits(),
             ctx_cfg.get_env(),
         )),
-        kernel_cmdline_epilog: Some(format!(" -- {}", ctx_cfg.get_args())),
+        epilog: Some(format!(" -- {}", ctx_cfg.get_args())),
     };
 
-    if ctx_cfg.vmr.set_boot_source(boot_source).is_err() {
+    if ctx_cfg.vmr.set_kernel_cmdline(kernel_cmdline).is_err() {
         return -libc::EINVAL;
     }
 
