@@ -20,9 +20,11 @@ use std::ffi::c_void;
 use std::fs::{read_link, File};
 use std::io::{Seek, SeekFrom};
 use std::mem::size_of;
+use std::os::fd::AsFd;
 use std::os::fd::AsRawFd;
 #[cfg(feature = "x")]
 use std::ptr;
+use std::ptr::NonNull;
 use std::sync::atomic::AtomicBool;
 #[cfg(feature = "x")]
 use std::sync::atomic::{AtomicU32, Ordering};
@@ -30,8 +32,9 @@ use std::sync::Arc;
 use std::sync::Condvar;
 use std::sync::Mutex;
 use std::thread;
-use zerocopy::AsBytes;
 use zerocopy::FromBytes;
+use zerocopy::Immutable;
+use zerocopy::IntoBytes;
 
 use crate::cross_domain::cross_domain_protocol::*;
 use crate::cross_domain::sys::channel;
@@ -49,11 +52,11 @@ use crate::rutabaga_core::RutabagaContext;
 use crate::rutabaga_core::RutabagaResource;
 use crate::rutabaga_os::SafeDescriptor;
 use crate::rutabaga_utils::*;
+use crate::DrmFormat;
 use crate::ImageAllocationInfo;
 use crate::ImageMemoryRequirements;
 use crate::RutabagaGralloc;
 use crate::RutabagaGrallocFlags;
-use crate::{AsRawDescriptor, DrmFormat};
 
 mod cross_domain_protocol;
 mod sys;
@@ -147,7 +150,7 @@ struct CrossDomainWorker {
 }
 
 #[allow(dead_code)] // Never used on macOS
-struct FutexPtr(*mut c_void);
+struct FutexPtr(NonNull<c_void>);
 unsafe impl Send for FutexPtr {}
 
 #[allow(dead_code)] // Never used on macOS
@@ -172,7 +175,7 @@ impl CrossDomainFutex {
         let uaddr2 = ptr::null::<()>();
         let val3 = 1u32;
         let address = address.0;
-        let atomic_val = unsafe { AtomicU32::from_ptr(address as *mut u32) };
+        let atomic_val = unsafe { AtomicU32::from_ptr(address.as_ptr() as *mut u32) };
         // The goal of this code is to ensure that the other side observes at least
         // the latest wake event along with the value that the futex had when that
         // wake event was signaled.
@@ -210,7 +213,7 @@ impl CrossDomainFutex {
 
     fn shutdown(&mut self) {
         self.shutdown.store(true, Ordering::SeqCst);
-        let atomic_val = unsafe { AtomicU32::from_ptr(self.address.0 as *mut u32) };
+        let atomic_val = unsafe { AtomicU32::from_ptr(self.address.0.as_ptr() as *mut u32) };
         let v = atomic_val.load(Ordering::SeqCst);
         atomic_val.store(!v, Ordering::SeqCst);
         unsafe {
@@ -343,7 +346,7 @@ impl CrossDomainState {
 
     fn write_to_ring<T>(&self, mut ring_write: RingWrite<T>, ring_id: u32) -> RutabagaResult<usize>
     where
-        T: FromBytes + AsBytes,
+        T: FromBytes + IntoBytes + Immutable,
     {
         let mut context_resources = self.context_resources.lock().unwrap();
         let mut bytes_read: usize = 0;
@@ -609,7 +612,7 @@ impl CrossDomainWorker {
                     add_item(&self.item_state, CrossDomainItem::ShmBlob(file.into()))
                 };
             } else {
-                let flags = fcntl(file.as_raw_descriptor(), FcntlArg::F_GETFL)?;
+                let flags = fcntl(&file, FcntlArg::F_GETFL)?;
                 *identifier_type = match flags & O_ACCMODE {
                     O_WRONLY => CROSS_DOMAIN_ID_TYPE_WRITE_PIPE,
                     _ => return Err(RutabagaError::InvalidCrossDomainItemType),
@@ -895,7 +898,7 @@ impl CrossDomainContext {
                 4.try_into().unwrap(),
                 ProtFlags::PROT_WRITE | ProtFlags::PROT_READ,
                 MapFlags::MAP_SHARED,
-                handle.as_raw_fd(),
+                handle.as_fd(),
                 0,
             )?
         };
@@ -916,7 +919,7 @@ impl CrossDomainContext {
         let shutdown2 = shutdown.clone();
         let fptr = FutexPtr(address);
         let initial_value =
-            unsafe { AtomicU32::from_ptr(address as *mut u32) }.load(Ordering::SeqCst);
+            unsafe { AtomicU32::from_ptr(address.as_ptr() as *mut u32) }.load(Ordering::SeqCst);
         let watcher_thread = Some(
             thread::Builder::new()
                 .name(format!("futexw {}", cmd_futex_new.id))
@@ -997,8 +1000,8 @@ impl CrossDomainContext {
         commands: &mut [u8],
     ) -> RutabagaResult<()> {
         let opaque_data_offset = size_of::<T>();
-        let mut cmd_send =
-            T::read_from_prefix(commands.as_bytes()).ok_or(RutabagaError::InvalidCommandBuffer)?;
+        let (mut cmd_send, _) = T::read_from_prefix(commands.as_bytes())
+            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
 
         let opaque_data = commands
             .get_mut(
@@ -1036,7 +1039,7 @@ impl Drop for CrossDomainContext {
 }
 
 #[repr(C)]
-#[derive(Copy, Clone, Default, AsBytes, FromBytes)]
+#[derive(Copy, Clone, Default, IntoBytes, Immutable, FromBytes)]
 struct CrossDomainInitVMinus1 {
     hdr: CrossDomainHeader,
     query_ring_id: u32,
@@ -1122,6 +1125,13 @@ impl RutabagaContext for CrossDomainContext {
         // Items that are removed from the table after one usage.
         match item {
             CrossDomainItem::ShmBlob(descriptor) => {
+                #[allow(unused_mut)]
+                let mut access = RUTABAGA_MAP_ACCESS_RW;
+                #[cfg(target_os = "linux")]
+                if fcntl(&descriptor, FcntlArg::F_GET_SEALS)? & libc::F_SEAL_WRITE != 0 {
+                    access &= !RUTABAGA_MAP_ACCESS_WRITE;
+                };
+
                 let hnd = RutabagaHandle {
                     os_handle: descriptor,
                     handle_type: RUTABAGA_MEM_HANDLE_TYPE_SHM,
@@ -1133,7 +1143,7 @@ impl RutabagaContext for CrossDomainContext {
                     blob: true,
                     blob_mem: resource_create_blob.blob_mem,
                     blob_flags: resource_create_blob.blob_flags,
-                    map_info: Some(RUTABAGA_MAP_CACHE_CACHED | RUTABAGA_MAP_ACCESS_RW),
+                    map_info: Some(RUTABAGA_MAP_CACHE_CACHED | access),
                     #[cfg(target_os = "macos")]
                     map_ptr: None,
                     info_2d: None,
@@ -1147,7 +1157,7 @@ impl RutabagaContext for CrossDomainContext {
             }
             CrossDomainItem::DmaBuf(descriptor) => {
                 let mut access = RUTABAGA_MAP_ACCESS_READ;
-                if fcntl(descriptor.as_raw_fd(), FcntlArg::F_GETFL)? & libc::O_WRONLY != 0 {
+                if fcntl(descriptor.as_fd(), FcntlArg::F_GETFL)? & libc::O_WRONLY != 0 {
                     access |= RUTABAGA_MAP_ACCESS_WRITE;
                 }
 
@@ -1184,27 +1194,28 @@ impl RutabagaContext for CrossDomainContext {
         }
 
         while !commands.is_empty() {
-            let hdr = CrossDomainHeader::read_from_prefix(commands.as_bytes())
-                .ok_or(RutabagaError::InvalidCommandBuffer)?;
+            let (hdr, _) = CrossDomainHeader::read_from_prefix(commands)
+                .map_err(|_e| RutabagaError::InvalidCommandBuffer)?;
 
             match hdr.cmd {
                 CROSS_DOMAIN_CMD_INIT => {
                     let cmd_init = CrossDomainInitV1::read_from_prefix(commands.as_bytes())
-                        .or_else(|| {
+                        .map(|(x, _)| x)
+                        .or_else(|_| {
                             CrossDomainInitV0::read_from_prefix(commands.as_bytes())
-                                .map(|x| x.upgrade())
+                                .map(|(x, _)| x.upgrade())
                         })
-                        .or_else(|| {
+                        .or_else(|_| {
                             CrossDomainInitVMinus1::read_from_prefix(commands.as_bytes())
-                                .map(|x| x.upgrade())
+                                .map(|(x, _)| x.upgrade())
                         })
-                        .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                        .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
                     self.initialize(&cmd_init)?;
                 }
                 CROSS_DOMAIN_CMD_GET_IMAGE_REQUIREMENTS => {
-                    let cmd_get_reqs =
+                    let (cmd_get_reqs, _) =
                         CrossDomainGetImageRequirements::read_from_prefix(commands.as_bytes())
-                            .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
 
                     self.get_image_requirements(&cmd_get_reqs)?;
                 }
@@ -1220,8 +1231,9 @@ impl RutabagaContext for CrossDomainContext {
                 }
                 CROSS_DOMAIN_CMD_WRITE => {
                     let opaque_data_offset = size_of::<CrossDomainReadWrite>();
-                    let cmd_write = CrossDomainReadWrite::read_from_prefix(commands.as_bytes())
-                        .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                    let (cmd_write, _) =
+                        CrossDomainReadWrite::read_from_prefix(commands.as_bytes())
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
 
                     let opaque_data = commands
                         .get_mut(
@@ -1236,28 +1248,29 @@ impl RutabagaContext for CrossDomainContext {
                 }
                 #[cfg(feature = "x")]
                 CROSS_DOMAIN_CMD_FUTEX_NEW => {
-                    let cmd_new_futex = CrossDomainFutexNew::read_from_prefix(commands.as_bytes())
-                        .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                    let (cmd_new_futex, _) =
+                        CrossDomainFutexNew::read_from_prefix(commands.as_bytes())
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
                     self.futex_new(&cmd_new_futex)?;
                 }
                 #[cfg(feature = "x")]
                 CROSS_DOMAIN_CMD_FUTEX_SIGNAL => {
-                    let cmd_futex_signal =
+                    let (cmd_futex_signal, _) =
                         CrossDomainFutexSignal::read_from_prefix(commands.as_bytes())
-                            .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
                     self.futex_signal(&cmd_futex_signal)?;
                 }
                 #[cfg(feature = "x")]
                 CROSS_DOMAIN_CMD_FUTEX_DESTROY => {
-                    let cmd_futex_destroy =
+                    let (cmd_futex_destroy, _) =
                         CrossDomainFutexDestroy::read_from_prefix(commands.as_bytes())
-                            .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
                     self.futex_destroy(&cmd_futex_destroy)?;
                 }
                 CROSS_DOMAIN_CMD_READ_EVENTFD_NEW => {
-                    let cmd_new_efd =
+                    let (cmd_new_efd, _) =
                         CrossDomainReadEventfdNew::read_from_prefix(commands.as_bytes())
-                            .ok_or(RutabagaError::InvalidCommandBuffer)?;
+                            .map_err(|_| RutabagaError::InvalidCommandBuffer)?;
                     self.read_eventfd_new(&cmd_new_efd)?;
                 }
                 _ => return Err(RutabagaError::SpecViolation("invalid cross domain command")),
