@@ -4,13 +4,11 @@
 
 use std::convert::TryInto;
 use std::ffi::CStr;
-use std::os::unix::io::AsRawFd;
-use std::os::unix::io::FromRawFd;
 use std::os::unix::io::OwnedFd;
 
 use libc::off_t;
 use nix::sys::memfd::memfd_create;
-use nix::sys::memfd::MemFdCreateFlag;
+use nix::sys::memfd::MFdFlags;
 use nix::unistd::ftruncate;
 use nix::unistd::sysconf;
 use nix::unistd::SysconfVar;
@@ -36,14 +34,13 @@ impl SharedMemory {
     /// The file descriptor is opened with the close on exec flag and allows memfd sealing.
     pub fn new(debug_name: &CStr, size: u64) -> RutabagaResult<SharedMemory> {
         // Nix will transition to owned fd in future releases, do it locally here.
-        let raw_fd = memfd_create(
+        let fd = memfd_create(
             debug_name,
-            MemFdCreateFlag::MFD_CLOEXEC | MemFdCreateFlag::MFD_ALLOW_SEALING,
+            MFdFlags::MFD_CLOEXEC | MFdFlags::MFD_ALLOW_SEALING,
         )?;
-        let fd = unsafe { OwnedFd::from_raw_fd(raw_fd) };
 
         let size_off_t: off_t = size.try_into()?;
-        ftruncate(fd.as_raw_fd(), size_off_t)?;
+        ftruncate(&fd, size_off_t)?;
 
         Ok(SharedMemory { fd, size })
     }
