@@ -14,6 +14,7 @@ use std::fs::File;
 use std::io::{self, Read};
 #[cfg(target_os = "linux")]
 use std::os::fd::AsRawFd;
+use std::os::fd::{BorrowedFd, FromRawFd};
 use std::path::PathBuf;
 use std::sync::atomic::AtomicI32;
 use std::sync::{Arc, Mutex};
@@ -23,14 +24,12 @@ use super::{Error, Vmm};
 #[cfg(target_arch = "x86_64")]
 use crate::device_manager::legacy::PortIODeviceManager;
 use crate::device_manager::mmio::MMIODeviceManager;
-use crate::resources::VmResources;
+use crate::resources::{ConsoleType, VmResources};
 use crate::vmm_config::external_kernel::{ExternalKernel, KernelFormat};
 #[cfg(feature = "net")]
 use crate::vmm_config::net::NetBuilder;
 #[cfg(all(target_os = "linux", target_arch = "riscv64"))]
 use devices::legacy::KvmAia;
-#[cfg(all(target_os = "linux", target_arch = "aarch64"))]
-use devices::legacy::KvmGicV3;
 #[cfg(target_arch = "x86_64")]
 use devices::legacy::KvmIoapic;
 use devices::legacy::Serial;
@@ -41,6 +40,8 @@ use devices::legacy::{GicV3, HvfGicV3};
 #[cfg(target_arch = "x86_64")]
 use devices::legacy::{IoApic, IrqChipT};
 use devices::legacy::{IrqChip, IrqChipDevice};
+#[cfg(all(target_os = "linux", target_arch = "aarch64"))]
+use devices::legacy::{KvmGicV2, KvmGicV3};
 use devices::virtio::{port_io, MmioTransport, PortDescription, VirtioDevice, Vsock};
 
 #[cfg(feature = "tee")]
@@ -54,9 +55,9 @@ use crate::signal_handler::register_sigwinch_handler;
 use crate::terminal::term_set_raw_mode;
 #[cfg(feature = "blk")]
 use crate::vmm_config::block::BlockBuilder;
-use crate::vmm_config::boot_source::DEFAULT_KERNEL_CMDLINE;
 #[cfg(not(any(feature = "tee", feature = "nitro")))]
 use crate::vmm_config::fs::FsDeviceConfig;
+use crate::vmm_config::kernel_cmdline::DEFAULT_KERNEL_CMDLINE;
 #[cfg(target_os = "linux")]
 use crate::vstate::KvmContext;
 #[cfg(all(target_os = "linux", feature = "tee"))]
@@ -95,7 +96,8 @@ use vm_memory::GuestMemory;
 use vm_memory::GuestRegionMmap;
 use vm_memory::{GuestAddress, GuestMemoryMmap};
 
-#[cfg(feature = "efi")]
+#[cfg(target_arch = "aarch64")]
+#[allow(dead_code)]
 static EDK2_BINARY: &[u8] = include_bytes!("../../../edk2/KRUN_EFI.silent.fd");
 
 /// Errors associated with starting the instance.
@@ -115,6 +117,10 @@ pub enum StartMicrovmError {
     ElfOpenKernel(io::Error),
     /// Cannot load the kernel into the VM.
     ElfLoadKernel(linux_loader::loader::Error),
+    /// The firmware can't be loaded into the provided memory address.
+    FirmwareInvalidAddress(vm_memory::GuestMemoryError),
+    /// Cannot read firmware contents from file.
+    FirmwareRead(io::Error),
     /// Memory regions are overlapping or mmap fails.
     GuestMemoryMmap(vm_memory::Error),
     /// The BZIP2 decoder couldn't decompress the kernel.
@@ -190,6 +196,8 @@ pub enum StartMicrovmError {
     RegisterFsSigwinch(kvm_ioctls::Error),
     /// Cannot initialize a MMIO Gpu device or add a device to the MMIO Bus.
     RegisterGpuDevice(device_manager::mmio::Error),
+    /// Cannot initialize a MMIO Input device or add a device to the MMIO Bus.
+    RegisterInputDevice(device_manager::mmio::Error),
     /// Cannot initialize a MMIO Network Device or add a device to the MMIO Bus.
     RegisterNetDevice(device_manager::mmio::Error),
     /// Cannot initialize a MMIO Rng device or add a device to the MMIO Bus.
@@ -242,6 +250,15 @@ impl Display for StartMicrovmError {
             ElfLoadKernel(ref err) => {
                 write!(f, "Cannot load the kernel into the VM: {err}")
             }
+            FirmwareInvalidAddress(ref err) => {
+                write!(
+                    f,
+                    "The firmware can't be loaded into the guest memory: {err}"
+                )
+            }
+            FirmwareRead(ref err) => {
+                write!(f, "Cannot read firmware contents from file: {err}")
+            }
             GuestMemoryMmap(ref err) => {
                 // Remove imbricated quotes from error message.
                 let mut err_msg = format!("{err:?}");
@@ -402,6 +419,14 @@ impl Display for StartMicrovmError {
                     "Cannot initialize a MMIO Gpu Device or add a device to the MMIO Bus. {err_msg}"
                 )
             }
+            RegisterInputDevice(ref err) => {
+                let mut err_msg = format!("{err}");
+                err_msg = err_msg.replace('\"', "");
+                write!(
+                    f,
+                    "Cannot initialize a MMIO Input Device or add a device to the MMIO Bus. {err_msg}"
+                )
+            }
             RegisterNetDevice(ref err) => {
                 let mut err_msg = format!("{err}");
                 err_msg = err_msg.replace('\"', "");
@@ -490,7 +515,7 @@ enum Payload {
     ExternalKernel(ExternalKernel),
     #[cfg(test)]
     Empty,
-    Efi,
+    Firmware,
     #[cfg(feature = "tee")]
     Tee,
 }
@@ -512,8 +537,8 @@ fn choose_payload(vm_resources: &VmResources) -> Result<Payload, StartMicrovmErr
         return Ok(Payload::KernelCopy);
     } else if let Some(external_kernel) = vm_resources.external_kernel() {
         Ok(Payload::ExternalKernel(external_kernel.clone()))
-    } else if cfg!(feature = "efi") {
-        Ok(Payload::Efi)
+    } else if cfg!(feature = "efi") || vm_resources.firmware_config.is_some() {
+        Ok(Payload::Firmware)
     } else {
         Err(StartMicrovmError::MissingKernelConfig)
     }
@@ -542,6 +567,7 @@ pub fn build_microvm(
         vm_resources,
         &payload,
     )?;
+
     let vcpu_config = vm_resources.vcpu_config();
 
     // Clone the command-line so that a failed boot doesn't pollute the original.
@@ -549,16 +575,31 @@ pub fn build_microvm(
     let mut kernel_cmdline = Cmdline::new(arch::CMDLINE_MAX_SIZE);
     if let Some(cmdline) = payload_config.kernel_cmdline {
         kernel_cmdline.insert_str(cmdline.as_str()).unwrap();
-    } else if let Some(cmdline) = &vm_resources.boot_config.kernel_cmdline_prolog {
+    } else if let Some(cmdline) = &vm_resources.kernel_cmdline.prolog {
         kernel_cmdline.insert_str(cmdline).unwrap();
     } else {
         kernel_cmdline.insert_str(DEFAULT_KERNEL_CMDLINE).unwrap();
     }
 
-    if let Some(cmdline) = &vm_resources.boot_config.kernel_cmdline_krun_env {
+    if let Some(cmdline) = &vm_resources.kernel_cmdline.krun_env {
         kernel_cmdline.insert_str(cmdline.as_str()).unwrap();
     }
 
+    if let Some(kernel_console) = &vm_resources.kernel_console {
+        let cmdline = kernel_cmdline.as_str();
+        let console_start_idx = cmdline.find("console=").unwrap();
+        let console_end_idx = cmdline
+            .get(console_start_idx..)
+            .and_then(|s| s.find(" ").map(|i| i + console_start_idx));
+
+        let cmdline = cmdline.replace(
+            &cmdline[console_start_idx..console_end_idx.unwrap()],
+            format!("console={kernel_console}").as_str(),
+        );
+        kernel_cmdline = Cmdline::new(arch::CMDLINE_MAX_SIZE);
+        kernel_cmdline.insert_str(cmdline).unwrap();
+    }
+
     #[cfg(not(feature = "tee"))]
     #[allow(unused_mut)]
     let mut vm = setup_vm(&guest_memory, vm_resources.nested_enabled)?;
@@ -622,9 +663,9 @@ pub fn build_microvm(
 
         vec![
             MeasuredRegion {
-                guest_addr: arch::BIOS_START,
+                guest_addr: arch::FIRMWARE_START,
                 host_addr: guest_memory
-                    .get_host_address(GuestAddress(arch::BIOS_START))
+                    .get_host_address(GuestAddress(arch::FIRMWARE_START))
                     .unwrap() as u64,
                 size: qboot_size,
             },
@@ -665,9 +706,9 @@ pub fn build_microvm(
                 size: 0x8000_0000,
             },
             MeasuredRegion {
-                guest_addr: arch::BIOS_START,
+                guest_addr: arch::FIRMWARE_START,
                 host_addr: guest_memory
-                    .get_host_address(GuestAddress(arch::BIOS_START))
+                    .get_host_address(GuestAddress(arch::FIRMWARE_START))
                     .unwrap() as u64,
                 size: qboot_size,
             },
@@ -676,20 +717,41 @@ pub fn build_microvm(
         m
     };
 
-    // On x86_64 always create a serial device,
-    // while on aarch64 only create it if 'console=' is specified in the boot args.
-    let serial_device = if cfg!(feature = "efi") {
-        Some(setup_serial_device(
+    let mut serial_devices = Vec::new();
+
+    // Create the legacy serial device if we're booting from a firmware
+    if (cfg!(feature = "efi") || vm_resources.firmware_config.is_some())
+        && !vm_resources.disable_implicit_console
+    {
+        serial_devices.push(setup_serial_device(
             event_manager,
             None,
             None,
             // Uncomment this to get EFI output when debugging EDK2.
             //Some(Box::new(io::stdout())),
-        )?)
-    } else {
-        None
+        )?);
     };
 
+    for s in vm_resources
+        .consoles
+        .get(&ConsoleType::Serial)
+        .unwrap_or(&Vec::new())
+    {
+        let input: Option<Box<dyn devices::legacy::ReadableFd + Send>> = if s.input_fd >= 0 {
+            Some(Box::new(unsafe { File::from_raw_fd(s.input_fd) }))
+        } else {
+            None
+        };
+
+        let output: Option<Box<dyn io::Write + Send>> = if s.output_fd >= 0 {
+            Some(Box::new(unsafe { File::from_raw_fd(s.output_fd) }))
+        } else {
+            None
+        };
+
+        serial_devices.push(setup_serial_device(event_manager, input, output)?);
+    }
+
     let exit_evt = EventFd::new(utils::eventfd::EFD_NONBLOCK)
         .map_err(Error::EventFd)
         .map_err(StartMicrovmError::Internal)?;
@@ -698,7 +760,7 @@ pub fn build_microvm(
     // Safe to unwrap 'serial_device' as it's always 'Some' on x86_64.
     // x86_64 uses the i8042 reset event as the Vmm exit event.
     let mut pio_device_manager = PortIODeviceManager::new(
-        serial_device,
+        serial_devices,
         exit_evt
             .try_clone()
             .map_err(Error::EventFd)
@@ -746,6 +808,8 @@ pub fn build_microvm(
             Some(intc.clone()),
         )?;
 
+        let kernel_boot = vm_resources.firmware_config.is_none() && !cfg!(feature = "tee");
+
         vcpus = create_vcpus_x86_64(
             &vm,
             &vcpu_config,
@@ -753,6 +817,7 @@ pub fn build_microvm(
             payload_config.entry_addr,
             &pio_device_manager.io_bus,
             &exit_evt,
+            kernel_boot,
             #[cfg(feature = "tee")]
             _sender,
         )
@@ -782,16 +847,27 @@ pub fn build_microvm(
         )
         .map_err(StartMicrovmError::Internal)?;
 
-        intc = Arc::new(Mutex::new(IrqChipDevice::new(Box::new(KvmGicV3::new(
-            vm.fd(),
-            vm_resources.vm_config().vcpu_count.unwrap() as u64,
-        )))));
+        intc = {
+            // The SoC in some popular boards (namely, the RPi family) doesn't support an
+            // architected vGIC, which is required for requesting KVM the instantiation of a
+            // GICv3. To relieve the users from having to configure the gic version manually,
+            // try first to instantiate a GICv3, and fall back to a GICv2 if it fails.
+            let vcpu_count = vm_resources.vm_config().vcpu_count.unwrap() as u64;
+            let gic = match KvmGicV3::new(vm.fd(), vcpu_count) {
+                Ok(gicv3) => IrqChipDevice::new(Box::new(gicv3)),
+                Err(_) => {
+                    warn!("KVM GICv3 creation failed, falling back to KVM GICv2");
+                    IrqChipDevice::new(Box::new(KvmGicV2::new(vm.fd(), vcpu_count)))
+                }
+            };
+            Arc::new(Mutex::new(gic))
+        };
 
         attach_legacy_devices(
             &vm,
             &mut mmio_device_manager,
             &mut kernel_cmdline,
-            serial_device,
+            serial_devices,
         )?;
     }
 
@@ -823,7 +899,7 @@ pub fn build_microvm(
             &mut mmio_device_manager,
             &mut kernel_cmdline,
             intc.clone(),
-            serial_device,
+            serial_devices,
             event_manager,
             _shutdown_efd,
         )?;
@@ -873,12 +949,35 @@ pub fn build_microvm(
     attach_balloon_device(&mut vmm, event_manager, intc.clone())?;
     #[cfg(not(feature = "tee"))]
     attach_rng_device(&mut vmm, event_manager, intc.clone())?;
-    attach_console_devices(
-        &mut vmm,
-        event_manager,
-        intc.clone(),
-        vm_resources.console_output.clone(),
-    )?;
+    let mut console_id = 0;
+    if !vm_resources.disable_implicit_console {
+        attach_console_devices(
+            &mut vmm,
+            event_manager,
+            intc.clone(),
+            vm_resources,
+            None,
+            console_id,
+        )?;
+        console_id += 1;
+    }
+
+    for console in vm_resources
+        .consoles
+        .get(&ConsoleType::Virtio)
+        .unwrap_or(&Vec::new())
+        .iter()
+    {
+        attach_console_devices(
+            &mut vmm,
+            event_manager,
+            intc.clone(),
+            vm_resources,
+            Some(console),
+            console_id,
+        )?;
+        console_id += 1;
+    }
 
     #[cfg(not(any(feature = "tee", feature = "nitro")))]
     let export_table: Option<ExportTable> = if cfg!(feature = "gpu") {
@@ -907,6 +1006,12 @@ pub fn build_microvm(
             _sender.clone(),
         )?;
     }
+
+    #[cfg(feature = "input")]
+    if !vm_resources.input_backends.is_empty() {
+        attach_input_devices(&mut vmm, &vm_resources.input_backends, intc.clone())?;
+    }
+
     #[cfg(not(any(feature = "tee", feature = "nitro")))]
     attach_fs_devices(
         &mut vmm,
@@ -938,7 +1043,7 @@ pub fn build_microvm(
         attach_snd_device(&mut vmm, intc.clone())?;
     }
 
-    if let Some(s) = &vm_resources.boot_config.kernel_cmdline_epilog {
+    if let Some(s) = &vm_resources.kernel_cmdline.epilog {
         vmm.kernel_cmdline.insert_str(s).unwrap();
     };
 
@@ -1250,7 +1355,7 @@ fn load_payload(
             let qboot_data =
                 unsafe { std::slice::from_raw_parts(qboot_host_addr as *mut u8, qboot_size) };
             guest_mem
-                .write(qboot_data, GuestAddress(arch::BIOS_START))
+                .write(qboot_data, GuestAddress(arch::FIRMWARE_START))
                 .unwrap();
 
             let (initrd_host_addr, initrd_size) =
@@ -1277,15 +1382,7 @@ fn load_payload(
                 None,
             ))
         }
-        #[cfg(feature = "efi")]
-        Payload::Efi => {
-            guest_mem.write(EDK2_BINARY, GuestAddress(0u64)).unwrap();
-            Ok((guest_mem, GuestAddress(0), None, None))
-        }
-        #[cfg(not(feature = "efi"))]
-        Payload::Efi => {
-            unreachable!("EFI support was not built in")
-        }
+        Payload::Firmware => Ok((guest_mem, GuestAddress(arch::RESET_VECTOR), None, None)),
     }
 }
 
@@ -1305,6 +1402,17 @@ fn create_guest_memory(
 > {
     let mem_size = mem_size << 20;
 
+    #[cfg(not(feature = "efi"))]
+    let (firmware_data, firmware_size) = if let Some(firmware) = &vm_resources.firmware_config {
+        let data = std::fs::read(firmware.path.clone()).map_err(StartMicrovmError::FirmwareRead)?;
+        let len = data.len();
+        (Some(data), Some(len))
+    } else {
+        (None, None)
+    };
+    #[cfg(feature = "efi")]
+    let (firmware_data, firmware_size) = (Some(EDK2_BINARY), Some(EDK2_BINARY.len()));
+
     #[cfg(target_arch = "x86_64")]
     let (arch_mem_info, mut arch_mem_regions) = match payload {
         #[cfg(not(feature = "tee"))]
@@ -1315,10 +1423,10 @@ fn create_guest_memory(
                 } else {
                     return Err(StartMicrovmError::MissingKernelConfig);
                 };
-            arch::arch_memory_regions(mem_size, Some(kernel_guest_addr), kernel_size, 0)
+            arch::arch_memory_regions(mem_size, Some(kernel_guest_addr), kernel_size, 0, None)
         }
         Payload::ExternalKernel(external_kernel) => {
-            arch::arch_memory_regions(mem_size, None, 0, external_kernel.initramfs_size)
+            arch::arch_memory_regions(mem_size, None, 0, external_kernel.initramfs_size, None)
         }
         #[cfg(feature = "tee")]
         Payload::Tee => {
@@ -1328,18 +1436,18 @@ fn create_guest_memory(
                 } else {
                     return Err(StartMicrovmError::MissingKernelConfig);
                 };
-            arch::arch_memory_regions(mem_size, Some(kernel_guest_addr), kernel_size, 0)
+            arch::arch_memory_regions(mem_size, Some(kernel_guest_addr), kernel_size, 0, None)
         }
         #[cfg(test)]
-        Payload::Empty => arch::arch_memory_regions(mem_size, None, 0, 0),
-        Payload::Efi => unreachable!(),
+        Payload::Empty => arch::arch_memory_regions(mem_size, None, 0, 0, None),
+        Payload::Firmware => arch::arch_memory_regions(mem_size, None, 0, 0, firmware_size),
     };
     #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     let (arch_mem_info, mut arch_mem_regions) = match payload {
         Payload::ExternalKernel(external_kernel) => {
-            arch::arch_memory_regions(mem_size, external_kernel.initramfs_size)
+            arch::arch_memory_regions(mem_size, external_kernel.initramfs_size, None)
         }
-        _ => arch::arch_memory_regions(mem_size, 0),
+        _ => arch::arch_memory_regions(mem_size, 0, firmware_size),
     };
 
     let mut shm_manager = ShmManager::new(&arch_mem_info);
@@ -1367,6 +1475,12 @@ fn create_guest_memory(
     let (guest_mem, entry_addr, initrd_config, cmdline) =
         load_payload(vm_resources, guest_mem, &arch_mem_info, payload)?;
 
+    if let Some(firmware_data) = firmware_data.as_ref() {
+        guest_mem
+            .write(firmware_data, GuestAddress(arch_mem_info.firmware_addr))
+            .map_err(StartMicrovmError::FirmwareInvalidAddress)?;
+    }
+
     let payload_config = PayloadConfig {
         entry_addr,
         initrd_config,
@@ -1495,8 +1609,10 @@ fn attach_legacy_devices(
         }};
     }
 
-    register_irqfd_evt!(com_evt_1_3, 4);
-    register_irqfd_evt!(com_evt_2_4, 3);
+    register_irqfd_evt!(com_evt_1, 4);
+    register_irqfd_evt!(com_evt_2, 3);
+    register_irqfd_evt!(com_evt_3, 4);
+    register_irqfd_evt!(com_evt_4, 3);
     register_irqfd_evt!(kbd_evt, 1);
     Ok(())
 }
@@ -1509,11 +1625,11 @@ fn attach_legacy_devices(
     vm: &Vm,
     mmio_device_manager: &mut MMIODeviceManager,
     kernel_cmdline: &mut kernel::cmdline::Cmdline,
-    serial: Option<Arc<Mutex<Serial>>>,
+    serial: Vec<Arc<Mutex<Serial>>>,
 ) -> std::result::Result<(), StartMicrovmError> {
-    if let Some(serial) = serial {
+    for s in serial {
         mmio_device_manager
-            .register_mmio_serial(vm.fd(), kernel_cmdline, serial)
+            .register_mmio_serial(vm.fd(), kernel_cmdline, s)
             .map_err(Error::RegisterMMIODevice)
             .map_err(StartMicrovmError::Internal)?;
     }
@@ -1533,13 +1649,13 @@ fn attach_legacy_devices(
     mmio_device_manager: &mut MMIODeviceManager,
     kernel_cmdline: &mut kernel::cmdline::Cmdline,
     intc: IrqChip,
-    serial: Option<Arc<Mutex<Serial>>>,
+    serial: Vec<Arc<Mutex<Serial>>>,
     event_manager: &mut EventManager,
     shutdown_efd: Option<EventFd>,
 ) -> Result<(), StartMicrovmError> {
-    if let Some(serial) = serial {
+    for s in serial {
         mmio_device_manager
-            .register_mmio_serial(vm, kernel_cmdline, intc.clone(), serial)
+            .register_mmio_serial(vm, kernel_cmdline, intc.clone(), s)
             .map_err(Error::RegisterMMIODevice)
             .map_err(StartMicrovmError::Internal)?;
     }
@@ -1565,6 +1681,7 @@ fn attach_legacy_devices(
 }
 
 #[cfg(target_arch = "x86_64")]
+#[allow(clippy::too_many_arguments)]
 fn create_vcpus_x86_64(
     vm: &Vm,
     vcpu_config: &VcpuConfig,
@@ -1572,6 +1689,7 @@ fn create_vcpus_x86_64(
     entry_addr: GuestAddress,
     io_bus: &devices::Bus,
     exit_evt: &EventFd,
+    kernel_boot: bool,
     #[cfg(feature = "tee")] pm_sender: Sender<WorkerMessage>,
 ) -> super::Result<Vec<Vcpu>> {
     let mut vcpus = Vec::with_capacity(vcpu_config.vcpu_count as usize);
@@ -1588,7 +1706,7 @@ fn create_vcpus_x86_64(
         )
         .map_err(Error::Vcpu)?;
 
-        vcpu.configure_x86_64(guest_mem, entry_addr, vcpu_config)
+        vcpu.configure_x86_64(guest_mem, entry_addr, vcpu_config, kernel_boot)
             .map_err(Error::Vcpu)?;
 
         vcpus.push(vcpu);
@@ -1773,20 +1891,39 @@ fn attach_console_devices(
     vmm: &mut Vmm,
     event_manager: &mut EventManager,
     intc: IrqChip,
-    console_output: Option<PathBuf>,
+    vm_resources: &VmResources,
+    cfg: Option<&super::resources::ConsoleConfig>,
+    id_number: u32,
 ) -> std::result::Result<(), StartMicrovmError> {
     use self::StartMicrovmError::*;
 
-    let ports = if let Some(console_output) = console_output {
-        let file = File::create(console_output.as_path()).map_err(OpenConsoleFile)?;
+    let mut console_output_path: Option<PathBuf> = None;
+
+    // we don't care about the console output that was set for the vm_resources
+    // if the implicit console is disabled
+    if let Some(path) = vm_resources.console_output.clone() {
+        // only set the console output for the implicit console
+        if !vm_resources.disable_implicit_console && cfg.is_none() {
+            console_output_path = Some(path)
+        }
+    }
+
+    let ports = if console_output_path.is_some() {
+        let file = File::create(console_output_path.unwrap()).map_err(OpenConsoleFile)?;
         vec![PortDescription::Console {
             input: Some(port_io::input_empty().unwrap()),
             output: Some(port_io::output_file(file).unwrap()),
         }]
     } else {
-        let stdin_is_terminal = isatty(STDIN_FILENO).unwrap_or(false);
-        let stdout_is_terminal = isatty(STDOUT_FILENO).unwrap_or(false);
-        let stderr_is_terminal = isatty(STDERR_FILENO).unwrap_or(false);
+        let (input_fd, output_fd, err_fd) = match cfg {
+            Some(c) => (c.input_fd, c.output_fd, c.err_fd),
+            None => (STDIN_FILENO, STDOUT_FILENO, STDERR_FILENO),
+        };
+        let stdin_is_terminal =
+            isatty(unsafe { BorrowedFd::borrow_raw(input_fd) }).unwrap_or(false);
+        let stdout_is_terminal =
+            isatty(unsafe { BorrowedFd::borrow_raw(output_fd) }).unwrap_or(false);
+        let stderr_is_terminal = isatty(unsafe { BorrowedFd::borrow_raw(err_fd) }).unwrap_or(false);
 
         if let Err(e) = term_set_raw_mode(!stdin_is_terminal) {
             log::error!("Failed to set terminal to raw mode: {e}")
@@ -1794,6 +1931,10 @@ fn attach_console_devices(
 
         let console_input = if stdin_is_terminal {
             Some(port_io::stdin().unwrap())
+        } else if input_fd < 0 {
+            Some(port_io::input_empty().unwrap())
+        } else if input_fd != STDIN_FILENO {
+            Some(port_io::input_to_raw_fd_dup(input_fd).unwrap())
         } else {
             #[cfg(target_os = "linux")]
             {
@@ -1808,6 +1949,8 @@ fn attach_console_devices(
 
         let console_output = if stdout_is_terminal {
             Some(port_io::stdout().unwrap())
+        } else if output_fd != STDOUT_FILENO && output_fd > 0 {
+            Some(port_io::output_to_raw_fd_dup(output_fd).unwrap())
         } else {
             Some(port_io::output_to_log_as_err())
         };
@@ -1817,21 +1960,34 @@ fn attach_console_devices(
             output: console_output,
         }];
 
-        if !stdin_is_terminal {
+        let console_err = if stderr_is_terminal {
+            Some(port_io::stderr().unwrap())
+        } else if err_fd != STDERR_FILENO && err_fd > 0 {
+            Some(port_io::output_to_raw_fd_dup(err_fd).unwrap())
+        } else {
+            None
+        };
+
+        ports.push(PortDescription::Console {
+            input: None,
+            output: console_err,
+        });
+
+        if !stdin_is_terminal && input_fd == STDIN_FILENO {
             ports.push(PortDescription::InputPipe {
                 name: "krun-stdin".into(),
                 input: port_io::stdin().unwrap(),
             })
         }
 
-        if !stdout_is_terminal {
+        if !stdout_is_terminal && output_fd == STDOUT_FILENO {
             ports.push(PortDescription::OutputPipe {
                 name: "krun-stdout".into(),
                 output: port_io::stdout().unwrap(),
             })
         };
 
-        if !stderr_is_terminal {
+        if !stderr_is_terminal && err_fd == STDERR_FILENO {
             ports.push(PortDescription::OutputPipe {
                 name: "krun-stderr".into(),
                 output: port_io::stderr().unwrap(),
@@ -1854,7 +2010,9 @@ fn attach_console_devices(
         .map_err(RegisterFsSigwinch)?;
 
     // The device mutex mustn't be locked here otherwise it will deadlock.
-    attach_mmio_device(vmm, "hvc0".to_string(), intc, console).map_err(RegisterConsoleDevice)?;
+    attach_mmio_device(vmm, format!("hvc{id_number}"), intc, console)
+        .map_err(RegisterConsoleDevice)?;
+
     Ok(())
 }
 
@@ -2009,6 +2167,29 @@ fn attach_gpu_device(
     Ok(())
 }
 
+#[cfg(feature = "input")]
+fn attach_input_devices(
+    vmm: &mut Vmm,
+    input_backends: &[(
+        krun_input::InputConfigBackend<'static>,
+        krun_input::InputEventProviderBackend<'static>,
+    )],
+    intc: IrqChip,
+) -> std::result::Result<(), StartMicrovmError> {
+    use self::StartMicrovmError::*;
+
+    for (index, (config_backend, events_backend)) in input_backends.iter().enumerate() {
+        let input_device = Arc::new(Mutex::new(
+            devices::virtio::input::Input::new(*config_backend, *events_backend).unwrap(),
+        ));
+
+        let id = format!("input{}", index);
+        attach_mmio_device(vmm, id, intc.clone(), input_device).map_err(RegisterInputDevice)?;
+    }
+
+    Ok(())
+}
+
 #[cfg(feature = "snd")]
 fn attach_snd_device(vmm: &mut Vmm, intc: IrqChip) -> std::result::Result<(), StartMicrovmError> {
     use self::StartMicrovmError::*;
@@ -2070,6 +2251,7 @@ pub mod tests {
             entry_addr,
             &bus,
             &EventFd::new(utils::eventfd::EFD_NONBLOCK).unwrap(),
+            true,
         )
         .unwrap();
         assert_eq!(vcpu_vec.len(), vcpu_count as usize);
