@@ -5,8 +5,11 @@
 #[cfg(feature = "blk")]
 pub mod block;
 
-/// Wrapper for configuring the microVM boot source.
-pub mod boot_source;
+/// Wrapper for configuring the firmware.
+pub mod firmware;
+
+/// Wrapper for configuring the kernel command line.
+pub mod kernel_cmdline;
 
 /// Wrapper for configuring an external kernel to be loaded in the microVM.
 pub mod external_kernel;
